from typing import Dict
import os
import sys

from jinja2 import Template
import click

from create_tomochain_masternode import __version__, envs,  templates


@click.command(help='Set up a TomoChain masternode by running one command.')
@click.argument('masternode_name', type=click.Path(
    file_okay=False,
    resolve_path=True
))
@click.option('--testnet', is_flag=True, help='Testnet instead of mainnet.')
@click.version_option(version=__version__)
def entrypoint(masternode_name: click.Path, testnet: bool) -> None:
    """Command line interface entrypoint"""
    env = envs.testnet if testnet else envs.mainnet
    masternode_path = os.path.join(os.getcwd(), masternode_name)
    click.echo(
        '\n'
        'Creating a new masternode in '
        f'{click.style(masternode_path, fg="green")}.'
        '\n'
    )
    answers = ask()
    compose_template = Template(templates.compose)
    compose_content = compose_template.render(**answers, **env)
    env_template = Template(templates.env)
    env_content = env_template.render(**answers, **env)
    if not os.path.exists(masternode_path):
        os.makedirs(masternode_path)
    with open(f'{masternode_path}/docker-compose.yml', 'w') as compose_file:
        print(compose_content, file=compose_file)
    with open(f'{masternode_path}/.env.yml', 'w') as env_file:
        print(env_content, file=env_file)


def ask() -> Dict[str, str]:
    """Prompt users for parameters"""
    answers = {}
    answers['name'] = click.prompt('[?] Name')
    answers['private_key'] = click.prompt('[?] Private key', hide_input=True)
    answers['address'] = click.prompt(
        '[?] Address',
        value_proc=lambda x: x.strip('0x')
    )
    answers['storage'] = click.prompt(
        '[?] Storage',
        type=click.Choice(['volume', 'path']),
        default='path'
    )
    answers['data'] = click.prompt(
        f'[?] Chaindata {answers["storage"]}',
        type=click.Path(
            exists=True,
            file_okay=False,
            resolve_path=True
        ) if answers["storage"] == 'path' else click.STRING
    )
    answers['expose_rpc'] = click.confirm(
        '[?] Expose RPC',
    )
    answers['expose_ws'] = click.confirm(
        '[?] Expose WebSocket',
    )
    return answers


if __name__ == '__main__':
    # frozen app fix
    if getattr(sys, 'frozen', False):
        entrypoint(sys.argv[1:])



compose = """version: "3.4"

services:

  masternode_{{ name }}:
    image: tomochain/node:stable
    environment:
      IDENTITY: $IDENTITY
      PRIVATE_KEY: $PRIVATE_KEY
      BOOTNODES: {{ bootnodes }}
      NETWORK_ID: {{ network_id }}
      VERBOSITY: 3
      NETSTATS_HOST: stats.tomochain.com
      NETSTATS_PORT: 443
      WS_SECRET: {{ ws_secret }}
    volumes:
      - $DATA:/tomochain/data
    ports:
      - 30303:30303/tcp
      - 30303:30303/udp
      {%- if expose_rpc %}
      - 8545:8545
      {%- endif %}
      {%- if expose_ws %}
      - 8546:8546
      {%- endif %}
    restart: always

  metrics_{{ name }}:
    image: tomochain/telegraf:stable
    hostname: $ADDRESS
    environment:
      METRICS_ENDPOINT: {{ metrics_endpoint }}
    volumes:
      - /var/run/docker.sock:/var/run/docker.sock:ro
      - /sys:/rootfs/sys:ro
      - /proc:/rootfs/proc:ro
      - /etc:/rootfs/etc:ro
    restart: always

{% if storage == "volume" -%}
volumes:
  {{ data }}:
{%- endif %}

"""



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['create_tomochain_masternode', 'create_tomochain_masternode.templates']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0', 'jinja2>=2.10,<3.0']

entry_points = \
{'console_scripts': ['create-tomochain-masternode = '
                     'create_tomochain_masternode.main:entrypoint']}

setup_kwargs = {
    'name': 'create-tomochain-masternode',
    'version': '0.1.1b0',
    'description': 'Set up a TomoChain masternode by running one command.',
    'long_description': '# create-tomochain-masternode\nSet up a modern web app by running one command.\n',
    'author': 'etienne-napoleone',
    'author_email': 'etienne@tomochain.com',
    'url': 'https://tomochain.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
